@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 # lesspipe.sh, a preprocessor for less
-lesspipe_version=2.15
+lesspipe_version=2.16
 # Author: Wolfgang Friebel (wp.friebel AT gmail.com)
 
 has_cmd () {
@@ -78,19 +78,14 @@ filetype () {
 			[[ $fcat == text ]] && ftype="$fext" ;;
 		tsx)
 			[[ $fcat == text ]] && ftype=typescript-jsx ;;
-		appimage)
-			[[ $fcat == application && -x "$1" ]] && ftype="$fext" ;;
+		appimage|AppImage)
+			[[ $fcat == application && -x "$1" ]] && ftype=appimage ;;
 		snap)
 			[[ $fcat == application ]] && ftype="$fext" ;;
 	esac
 	### get file type from 'file' command for an unspecific result
-	if [[ "$fcat" == message && $ftype == plain ]]; then
-		ftype=msg
-	fi
-	if [[ "$fcat" == message && $ftype == rfc822 ]]; then
-		fcat=text
-		ftype=email
-	fi
+	[[ "$fcat" == message && $ftype == plain ]] && ftype=msg
+	[[ "$fcat" == message && $ftype == rfc822 ]] && fcat=text && ftype=email
 	if [[ "$fcat" == application && "$ftype" == octet-stream || "$fcat" == text && $ftype == plain ]]; then
 		ft=$(file -L -s -b "$1" 2> /dev/null)
 		# first check if the file command yields something
@@ -206,30 +201,22 @@ show () {
 	file1="${1%%"$sep"*}"
 	rest1="${1#"$file1"}"
 	while [[ "$rest1" == "$sep$sep"* ]]; do
-		if [[ "$rest1" == "$sep$sep" ]]; then
-			break
-		else
-			rest1="${rest1#"$sep$sep"}"
-			file1="${rest1%%"$sep"*}"
-			rest1="${rest1#"$file1"}"
-			file1="${1%"$rest1"}"
-		fi
+		[[ "$rest1" == "$sep$sep" ]] && break
+		rest1="${rest1#"$sep$sep"}"
+		file1="${rest1%%"$sep"*}"
+		rest1="${rest1#"$file1"}"
+		file1="${1%"$rest1"}"
 	done
-	if [[ ! -e "$file1" && "$file1" != '-' ]]; then
-		exit 1
-	fi
+	[[ ! -e "$file1" && "$file1" != '-' ]] && exit 1
 	rest11="${rest1#"$sep"}"
 	file2="${rest11%%"$sep"*}"
 	rest2="${rest11#"$file2"}"
 	while [[ "$rest2" == "$sep$sep"* ]]; do
-		if [[ "$rest2" == "$sep$sep" ]]; then
-			break
-		else
-			rest2="${rest2#"$sep$sep"}"
-			file2="${rest2%%"$sep"*}"
-			rest2="${rest2#"$file2"}"
-			file2="${rest11%"$rest2"}"
-		fi
+		[[ "$rest2" == "$sep$sep" ]] && break
+		rest2="${rest2#"$sep$sep"}"
+		file2="${rest2%%"$sep"*}"
+		rest2="${rest2#"$file2"}"
+		file2="${rest11%"$rest2"}"
 	done
 	rest2="${rest11#"$file2"}"
 	rest11="$rest1"
@@ -303,9 +290,7 @@ get_unpack_cmd () {
 	fcat="${1##*:}"
 	x="${1%%:*}"
 	cmd=()
-	if [[ "$3" == $sep$sep ]]; then
-		return
-	fi
+	[[ "$3" == $sep$sep ]] && return
 	declare t
 	# uncompress / transform
 	case $x in
@@ -314,43 +299,34 @@ get_unpack_cmd () {
 			[[ $2 == - ]] || fileext="$2"
 			fileext=${fileext%%.gz}; fileext=${fileext%%.bz2}
 			[[ $x == compress ]] && x=gzip
-			has_cmd "$x" && cmd=("$x" -cd "$2") && return ;;
+			has_cmd "$x" && cmd=("$x" -cd "$2") ;;
 		zstd)
-			has_cmd zstd && cmd=(zstd -cdqM1073741824 "$2") && return ;;
+			has_cmd zstd && cmd=(zstd -cdqM1073741824 "$2") ;;
 		lz4)
-			has_cmd lz4 && cmd=(lz4 -cdq "$2") && return ;;
+			has_cmd lz4 && cmd=(lz4 -cdq "$2") ;;
 		xlsx)
-			has_cmd in2csv && cmd=(in2csv -f xlsx "$2") && return
-			has_cmd excel2csv && cmd=(istemp excel2csv "$2") && return ;;
+			{ has_cmd in2csv && cmd=(in2csv -f xls "$2"); } ||
+			{ has_cmd excel2csv && cmd=(istemp xls2csv "$2"); } ;;
+		# xls2csv 0.95 fails test #72 xls (old) - define function to allow options
 		ms-excel)
-			has_cmd in2csv && cmd=(in2csv -f xls "$2") && return
-# xls2csv 0.95 fails test #72 xls (old) - define function to allow options
-			has_cmd xls2csv && cmd=(istemp xls2csv "$2") && return ;;
+			{ has_cmd in2csv && cmd=(in2csv -f xls "$2"); } ||
+			{ has_cmd xls2csv && cmd=(istemp xls2csv "$2"); } ;;
 	esac
+	[[ ${cmd[*]} == '' ]] || return
 	# convert into utf8
 
-	if [[ -n $lclocale && $fchar != binary && $fchar != *ascii && $fchar != "$lclocale" && $fchar != unknown* ]]; then
+	if [[ -n $charmap && $fchar != binary && $fchar != *ascii && $fchar != "$charmap" && $fchar != unknown* ]]; then
 		qm="\033[7m?\033[m" # inverted question mark
 		rep=(-c)
 		trans=()
-		echo ""|iconv --byte-subst - 2>/dev/null && rep=(--unicode-subst="$qm" --byte-subst="$qm" --widechar-subst="$qm") # MacOS
-# iconv 1.17 fails test #57 latin1: this also seems to fix
-# test #90 directory
-# iconv 1.17 supports //TRANSLIT only with charset encoding/code page
-# use lclocale charset encoding/code page instead of locale prefixed by ll_TT
-#		echo ""|iconv -f "$fchar" -t "$locale//TRANSLIT" - 2>/dev/null && trans=(-t "$locale//TRANSLIT")
-		echo ""|iconv -f "$fchar" -t "$lclocale//TRANSLIT" - 2>/dev/null && trans=(-t "$lclocale//TRANSLIT")
+		iconv --byte-subst - </dev/null 2>/dev/null && rep=(--unicode-subst="$qm" --byte-subst="$qm" --widechar-subst="$qm") # MacOS
+		iconv -f "$fchar" -t "$charmap//TRANSLIT" - </dev/null 2>/dev/null && trans=(-t "$charmap//TRANSLIT")
 		msg "append $sep$sep to filename to view the original $fchar encoded file"
-# iconv 1.17 does not support inputfile "-" => /dev/stdin, invalid option flag
-# change "-" to end args "--" not just null arg "" (causes other problems)
-		two="$2"
-		[ "$2" = "-" ] && two=--
-# iconv 1.17 fails test #56 utf-16, -f utf-XX[bl]e => -t utf-8 (with BOM)
-# -f utf-XX => -t utf-8 (without BOM): strip be/le suffix
-#		cmd=(iconv "${rep[@]}" -f "$fchar" "${trans[@]}" "$2") 
-		cmd=(iconv "${rep[@]}" -f "${fchar%[bl]e}" "${trans[@]}" "$two")
+	# iconv 1.17 fails test #56 utf-16, -f utf-XX[bl]e => -t utf-8 (with BOM)
+	#		-f utf-XX => -t utf-8 (without BOM): strip be/le suffix
+		cmd=(iconv "${rep[@]}" -f "${fchar%[bl]e}" "${trans[@]}" "$2")
 		# loop protection, just in case
-		lclocale=
+		charmap=
 		return
 	fi
 	[[ "$3" == "$sep" ]] && return
@@ -403,6 +379,11 @@ get_unpack_cmd () {
 				{ has_cmd 7za && prog=7za; } ;;
 		esac
 	fi
+	if [[ "$prog" = ar && "$2" = *@* ]]; then
+		t=$(nexttmp)
+		cat "$2" > "$t"
+		set "$2" "$t"
+	fi
 	[[ -n $prog ]] && cmd=(isarchive "$prog" "$2" "$file2")
 	if [[ -n ${cmd[*]} ]]; then
 		[[ -n "$file2" ]] && file2= && return
@@ -495,12 +476,8 @@ has_colorizer () {
 			[[ -n "$3" ]] && opt=(-l "$3" "$1") ;;
 		nvimpager)
 			opt=(-c "$1")
-			if [[ -n "$3" ]]; then
-				ft=${3##*/}
-				ft=${ft##*.}
-				opt=(-c "$1" --cmd "set filetype=$ft")
-			fi
-			;;
+			[[ -n "$3" ]] && ft=${3##*/} && ft=${ft##*.} &&
+				opt=(-c "$1" --cmd "set filetype=$ft") ;;
 		*)
 			return ;;
 	esac
@@ -547,7 +524,7 @@ isfinal () {
 			# filename needs to end in .class
 			has_cmd procyon && t=$t.class && cat "$1" > "$t" && cmd=(procyon "$t") ;;
 		markdown)
-			[[ $COLOR = *always ]] && mdopt=() || mdopt=(-c)
+			[[ $COLOR = *always ]] && mdopt=(--ansi ) || mdopt=(-c)
 			{ has_cmd mdcat && cmd=(mdcat "${mdopt[@]}" "$1"); } ||
 			{ has_cmd pandoc && cmd=(pandoc -t plain "$1"); } ;;
 		docx)
@@ -589,10 +566,9 @@ isfinal () {
 			declare macro=andoc
 			[[ "$fext" == me ]] && macro=e
 			[[ "$fext" == ms ]] && macro=s
+	# groff 1.23 fails test #74 as it produces escape codes: disable with -a
 			{ has_cmd mandoc && cmd=(nodash mandoc "$1"); } ||
 			{ has_cmd man && cmd=(man -l "$1"); } ||
-# groff 1.23 fails test #74 as it produces escape codes: disable with -a
-#			{ [[ $COLOR == *always ]] && has_cmd groff && cmd=(groff -s -p -t -e -Tutf8 -m "$macro" "$1"); } ;;
 			{ [[ $COLOR == *always ]] && has_cmd groff && cmd=(groff -a -s -p -t -e -Tutf8 -m "$macro" "$1"); } ;;
 		rtf)
 			{ has_cmd unrtf && cmd=(istemp "unrtf --text" "$1"); } ||
@@ -662,10 +638,7 @@ isfinal () {
 	if [[ -n ${cmd[*]} ]]; then
 		# TAU: When cmd starts with environment variable settings, bash will refuse to execute it via : "${cmd[@]}"
 		# The remedy is simple : Just run it through the "env" command in that case.
-		if [[ "$cmd" =~ '=' ]]; then
-			cmd=(env "${cmd[@]}")
-		fi
-		#[[ -n ${colorizer[*]} ]] && "${cmd[@]}" | "${colorizer[@]}" && return
+		[[ "$cmd" =~ '=' ]] && cmd=(env "${cmd[@]}")
 		"${cmd[@]}"
 	else
 		[[ -n ${colorizer[*]} && $fcat != binary ]] && "${colorizer[@]}" && return
@@ -726,11 +699,7 @@ isarchive () {
 				separatorline
 				isoinfo -fR"$joliet" -i "$t" ;;
 			cpio)
-				if [[ "$2" == - ]]; then
-					cpio -tv --quiet
-				else
-					cpio -tv --quiet --file "$2"
-				fi ;;
+				cpio -tv --quiet  < "$2" ;;
 			7zz|7za|7zr)
 				istemp "$prog l" "$2"
 		esac
@@ -751,10 +720,7 @@ isimage () {
 		offset="-o $("$2" --appimage-offset)"
 	fi
 	if [[ -z "$3" ]]; then
-		if [[ "$1" == snap ]]; then
-			has_cmd snap && snap info "$2"
-			separatorline
-		fi
+		[[ "$1" == snap ]] && has_cmd snap && snap info "$2" && separatorline
 		istemp "unsquashfs -d . -llc $offset" "$2"
 	else
 		istemp "unsquashfs -cat $offset" "$2" "$3"
@@ -797,6 +763,11 @@ isdeb () {
 			bsdtar xOf "$1" "$data" | bsdtar xOf - "$2"
 		fi
 	else
+		if [[ "$1" = *@* ]]; then
+			t=$(nexttmp)
+			cat "$1" > "$t"
+			set "$1" "$t"
+		fi
 		data=$(ar t "$1"|grep data)
 		ft=$(ar p "$1" "$data" | filetype -)
 		get_unpack_cmd "$ft" -
@@ -866,7 +837,7 @@ handle_w3m () {
 ishtml () {
 	[[ $1 == - ]] && arg1=-stdin || arg1="$1"
 	htmlopt=--unicode-snob
-	has_cmd html2text && echo ""|html2text -utf8 2>/dev/null && htmlopt=-utf8
+	has_cmd html2text && html2text -utf8 </dev/null 2>/dev/null && htmlopt=-utf8
 	# 3 lines following can easily be reshuffled according to the preferred tool
 	has_cmd elinks && nodash "elinks -dump -force-html" "$1" && return ||
 	has_cmd w3m && handle_w3m "$1" && return ||
@@ -881,8 +852,9 @@ set +o noclobber
 setopt sh_word_split 2>/dev/null
 PATH=$PATH:${0%%/lesspipe.sh}
 # the current locale in lowercase (or generic utf-8)
-locale=$(locale|grep LC_CTYPE|tr -d '"') || locale=utf-8
-lclocale=$(echo "${locale##*.}"|tr '[:upper:]' '[:lower:]')
+declare -l charmap
+charmap=$(locale -k  charmap) || charmap="charmap=utf-8"
+eval "$charmap"
 
 sep=:					# file name separator
 altsep='='				# alternate separator character
@@ -918,9 +890,8 @@ if [[ -z "$1" && "$0" == */lesspipe.sh ]]; then
 		echo "export LESSOPEN"
 	fi
 else
-	if [ -x "${HOME}/.lessfilter" ]; then
-		"${HOME}/.lessfilter" "$1" && exit 0
-	elif has_cmd lessfilter; then
+	[[ -x "${HOME}/.lessfilter" ]] && "${HOME}/.lessfilter" "$1" && exit 0
+	if has_cmd lessfilter; then
 		lessfilter "$1" && exit 0
 	fi
 	if [[ -z "$1" ]]; then
